/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <configuration.h>

const struct option vpzone_config_opts[] = {
	{"config", 1, NULL, 'f'},
	{"help",   0, NULL, 'h'},
	{NULL,     0, NULL,  0 },
};

void parse_configuration(char *file_name, ptable_t *cmd_set)
{
	char line[CMD_LINE_MAX_LENGTH];
	FILE *config_file = NULL;
	char **cmd;
	int lineno = 1;

	if ((config_file = fopen(file_name, "r")) == NULL)
	{
		DEBUG_MSG("Configuration Filename: %s\n", file_name);
		ERR("parse_configuration()");
		return;
	}

	while (!feof(config_file))
	{
		memset(line, '\0', CMD_LINE_MAX_LENGTH);

		if(fgets(line, CMD_LINE_MAX_LENGTH, config_file) == NULL)
			break;

		if(strchr(line, '\n') == NULL)
		{
			log_msg(ERROR_LVL, "%s - line %d - line of more than CMD_LINE_MAX_LENGTH chars!\n", file_name, lineno);
			return;
		}

		DEBUG_MSG("Line to parse: %s", line);

		if ((cmd = parse_command(line)) == NULL)
		{
			lineno++;
			continue;
		}

		switch (interpret_command(cmd, CMD_NONE, cmd_set))
		{
		case CMD_UNKNOWN:
			DEBUG_MSG("%s - line %d - Unknown command\n", file_name, lineno);
			break;
		case CMD_ERROR:
			log_msg(ERROR_LVL, "%s - line %d - Command raised to an error\n", file_name, lineno);
			break;
		case CMD_SUCCESS:
		default:
			break;
		}

		free_cmd_line(cmd);
		lineno++;
	}

	fclose(config_file);
}
