/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <clear.h>

static int manage_error_mnode_clear(mnode_t *mnode, short event, int s_ret, int err_code);

PLUGIN_DEPENDANCIES(NULL);

PLUGIN_GBL_VARS;

cipher_t clear[] = {
	{
		-1,
		"clear",
		NULL,
		NULL,
		NULL,
		NULL,
		recv_mnode_clear,
		send_mnode_clear
	},
	{-1, "", NULL, NULL, NULL, NULL, NULL, NULL }
};

plugin_type_t init(int plugin_id, in_plugin_handlers_t *iph)
{
	PLUGIN_SET_VARS(plugin_id, iph);

	clear[0].plugin_id = plugin_id;

	log_msg(INFO_LVL, "Loaded CLEAR plugin\n");

	return PLUGIN_CIPHER;
}

int unload(void)
{
	return 0;
}

cipher_t *get_ciphers(void)
{
	return clear;
}

void recv_mnode_clear(int sock, short event, void *arg)
{
	int in_bytes = 0;
	mnode_t *mnode = (mnode_t *)arg;
	int r_bytes = 0;

	/*Make compiler happy*/
	if ((sock != mnode->sock) || (event != EV_READ))
		return;
    /***/

	if (mnode->r_bytes == mdevice_g->buffer_size)
	{
		DEBUG_MSG("Clear - Input buffer full\n");
		return manage_packet(mnode);
	}

	r_bytes = 4096;
	if ((mdevice_g->buffer_size - mnode->r_bytes) < 4096)
		r_bytes = mdevice_g->buffer_size - mnode->r_bytes;

	in_bytes =  recv(mnode->sock, mnode->r_buffer+mnode->r_bytes, r_bytes, 0);

	if (in_bytes <= 0)
	{
		if (manage_error_mnode_clear(mnode, event, in_bytes, errno) < 0)
			remove_mnode(mdevice_g, mnode);
		return;
	}

	mnode->r_bytes += in_bytes;
	mnode->bytes_recv += in_bytes;

	manage_packet(mnode);
}

void send_mnode_clear(int sock, short event, void *arg)
{
	int out_bytes = 0;
	mnode_t *mnode = (mnode_t *)arg;
	int s_bytes = 0;

	/*Make compiler happy*/
	if ((sock != mnode->sock) || (event != EV_WRITE))
		return;
    /***/

	DEBUG_MSG("CLEAR Sending\n");

	s_bytes = mnode->w_bytes;

	if (mnode->w_rests > 0)
		s_bytes = mnode->w_rests;

	out_bytes = send(mnode->sock, mnode->w_buffer, (size_t) mnode->w_bytes, 0);

	if (out_bytes <= 0)
	{
		mnode->w_rests = s_bytes;
		if (manage_error_mnode_clear(mnode, event, out_bytes, errno) < 0)
			remove_mnode(mdevice_g, mnode);
		return;
	}

	if (mnode->w_rests > 0)
		mnode->w_rests = 0;

	mnode->w_bytes -= out_bytes;
	mnode->bytes_send += out_bytes;

	if (mnode->w_bytes <= 0)
	{
		mnode->w_bytes = 0;
		event_del(&mnode->write_event);
	}
	else
		memmove(mnode->w_buffer, mnode->w_buffer + out_bytes, (size_t) mnode->w_bytes);
}

static int manage_error_mnode_clear(mnode_t *mnode, short event, int s_ret, int err_code)
{
	if (s_ret == 0)
		return -1;
	
	DEBUG_MSG("CLEAR Send/Recv error code %d\n", err_code);

	switch(err_code)
	{
	case EINTR:
	case EAGAIN:
		if (event == EV_WRITE)
			event_add(&mnode->write_event, NULL);
		break;
	default:
		return -1;
	}

	return 0;
}
