/*    Copyright (C) 2010  Tomasz Bursztyka
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __VPZONE_PLUGIN_H__
#define __VPZONE_PLUGIN_H__

#include <config.h>

#include <vpzone.h>

#include <string.h>
#include <errno.h>

/**
 * A macro to define plugin dependancies.
 * If a plugin requires well known other plugins, it will try to load the
 * plugins recognized by their names.  Format is "name1 name2 " ...  This
 * is mandatory set at the beginning of main plugin source file.
 * @param deps NULL if no dependancies or a string made of plugin name(s)
 * separated by spaces.
 */
#define PLUGIN_DEPENDANCIES(deps)				\
	const char *dependencies = deps;

/**
 * A macro to define internal glogal variables. 
 * It provides  imported handlers variable from  the core as  well as the
 * plugin id  variable.  This if mandatory  set at the  beginning of main
 * plugin source file.
 * @see in_plugin_handlers_t
 */
#define PLUGIN_GBL_VARS							\
	in_plugin_handlers_t *ph = NULL;			\
	int p_id = 0;

/**
 * A macro to set internal global variables values.
 * This is mandatory to call this macros at first statement in the init()
 * function.   It  will  set  the   right  values  to  variables  set  by
 * PLUGIN_GBL_VARS.
 * @param i the  plugin id, you must use the one  given as first argument
 * it init()
 * @param p the imported handlers variable, you must use the one given as
 * second argument it init()
 * @see plugin_init_f
 */
#define PLUGIN_SET_VARS(i, p) \
	ph = p;					  \
	p_id = i;

/**
 * A macro to access the imported singleton mdevice data.
 * @see mdevice_t
 */
#define mdevice_g ph->mdevice_g


/******************	\
* Imported Handlers *
\*******************/
#define manage_packet ((manage_packet_f) ph->handlers[HDL_MANAGE_PACKET])

#define route_packet ((route_packet_f) ph->handlers[HDL_ROUTE_PACKET])

#define read_tun ((read_tun_f) ph->handlers[HDL_READ_DEVICE])

#define remove_mnode ((remove_mnode_f) ph->handlers[HDL_REMOVE_MNODE])

#define remove_mserver ((remove_mserver_f) ph->handlers[HDL_REMOVE_MSERVER])

#define remove_mdevice ((remove_mdevice_f) ph->handlers[HDL_REMOVE_MDEVICE])

#define select_route(ret_var, ...) HDL_RET(ret_var, NULL, select_route_f, \
										   ph->handlers[HDL_SELECT_ROUTE], __VA_ARGS__)

/******************	\
* Imported Handlers *
\*******************/
#define manage_internal_packet(ret_var, cb, ...) CB_RET(manage_internal_packet_f, cb, \
														ret_var, 0, __VA_ARGS__)

#define post_accept(cb, ...) CB_VOID(post_accept_f, cb, __VA_ARGS__) 

#define post_connect(cb, ...) CB_VOID(post_connect_f, cb, __VA_ARGS__)

#define pre_disconnect(cb, ...) CB_VOID(pre_disconnect_f, cb, __VA_ARGS__)

#define init_mnode_extent(cb, ...) CB_VOID(init_mnode_extent_f, cb, __VA_ARGS__)

#define remove_mnode_extent(cb, ...) CB_VOID(remove_mnode_extent_f,	\
											 cb, __VA_ARGS__)

#define init_mserver_extent(cb, ...) CB_VOID(init_mserver_extent_f,	\
											 cb, __VA_ARGS__)

#define remove_mserver_extent(cb, ...) CB_VOID(remove_mserver_extent_f, \
											   cb, __VA_ARGS__)

/*Connector specific*/
#define setup_mserver(m_d, m_s) ((setup_mserver_f) ((connector_t *) ph->connectors->ptable[m_s->connector_id])->setup_mserver_h)(m_d, m_s)
#define close_mserver(m_d, m_s) ((setup_mserver_f) ((connector_t *) ph->connectors->ptable[m_s->connector_id])->close_mserver_h)(m_d, m_s)
#define connect_mnode(m_d, m_n) ((connect_mnode_f) ((connector_t *) ph->connectors->ptable[m_n->connector_id])->connect_mnode_h)(m_d, m_n)
#define disconnect_mnode(m_n)   ((disconnect_mnode_f) ((connector_t *) ph->connectors->ptable[m_n->connector_id])->disconnect_mnode_h)(m_n)

/*Cipher specific*/
#define setup_mnode_server_cipher(ret_var, m_n)							\
	if ((m_n->cipher_id >= 0) &&										\
		(((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->setup_mnode_server_cipher_h != NULL)) \
		ret_var = ((setup_mnode_server_cipher_f)						\
				   ((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->setup_mnode_server_cipher_h)(m_n); \
	else																\
		ret_var = 1;

#define setup_mnode_client_cipher(ret_var, m_n)							\
	if ((m_n->cipher_id >= 0) &&										\
		(((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->setup_mnode_client_cipher_h != NULL)) \
		ret_var = ((setup_mnode_client_cipher_f)						\
				   ((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->setup_mnode_client_cipher_h)(m_n); \
	else																\
		ret_var = 1;

#define deinit_mnode_server_cipher(ret_var, m_n)						\
	if ((m_n->cipher_id >= 0) &&										\
		(((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->deinit_mnode_server_cipher_h != NULL)) \
		ret_var = ((deinit_mnode_server_cipher_f)						\
				   ((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->deinit_mnode_server_cipher_h)(m_n); \
	else																\
		ret_var = 1;

#define deinit_mnode_client_cipher(ret_var, m_n)						\
	if ((m_n->cipher_id >= 0) &&										\
		(((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->deinit_mnode_client_cipher_h != NULL)) \
		ret_var = ((deinit_mnode_client_cipher_f)						\
				   ((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->deinit_mnode_client_cipher_h)(m_n); \
	else																\
		ret_var = 1;

#define recv_mnode(s, e, m_n)								\
	((recv_mnode_f)														\
	 ((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->recv_mnode_h)(s, e, m_n)

#define recv_mnode_h(m_n)						\
	(recv_mnode_f)												\
	((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->recv_mnode_h

#define send_mnode(s, e, m_n)								\
	((send_mnode_f)														\
	 ((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->send_mnode_h)(s, e, m_n)

#define send_mnode_h(m_n)						\
	(send_mnode_f)												\
	((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->send_mnode_h

/*Authenticators specific*/
#define setup_mnode_server_authentification(ret_var, m_n)				\
	if ((m_n->authenticator_id >= 0) &&									\
		(((authenticator_t *) ph->authenticators->ptable[m_n->authenticator_id])->setup_mnode_server_authentification_h != NULL)) \
		ret_var = ((setup_mnode_server_authentification_f)				\
				   ((authenticator_t *) ph->authenticators->ptable[m_n->authenticator_id])->setup_mnode_server_authentification_h)(m_n); \
	else																\
		ret_var = 1;

#define setup_mnode_client_authentification(ret_var, m_n)				\
	if ((m_n->authenticator_id >= 0) &&									\
		(((authenticator_t *) ph->authenticators->ptable[m_n->authenticator_id])->setup_mnode_client_authentification_h != NULL)) \
		ret_var = ((setup_mnode_client_authentification_f)				\
				   ((authenticator_t *) ph->authenticators->ptable[m_n->authenticator_id])->setup_mnode_client_authentification_h)(m_n); \
	else																\
		ret_var = 1;

#define deinit_mnode_server_authentification(ret_var, m_n)				\
	if ((m_n->authenticator_id >= 0) &&									\
		(((authenticator_t *) ph->authenticators->ptable[m_n->authenticator_id])->deinit_mnode_server_authentification_h != NULL)) \
		ret_var = ((deinit_mnode_server_authentification_f)				\
				   ((authenticator_t *) ph->authenticators->ptable[m_n->authenticator_id])->deinit_mnode_server_authentification_h)(m_n); \
	else																\
		ret_var = 1;

#define deinit_mnode_client_authentification(ret_var, m_n)				\
	if ((m_n->authenticator_id >= 0) &&									\
		(((authenticator_t *) ph->authenticators->ptable[m_n->authenticator_id])->deinit_mnode_client_authentification_h != NULL)) \
		ret_var = ((deinit_mnode_client_authentification_f)				\
				   ((authenticator_t *) ph->authenticators->ptable[m_n->authenticator_id])->deinit_mnode_client_authentification_h)(m_n); \
	else																\
		ret_var = 1;

/*******\
* Utils *
\*******/
#define log_msg ph->log_msg_h
#define log_set_redirection ph->log_set_redirection_h
#define log_unset_redirection ph->log_unset_redirection_h
#define set_handler ph->set_handler_h
#define unset_handler ph->unset_handler_h
#define add_callback ph->add_callback_h
#define sub_callback ph->sub_callback_h
#define init_mnode ph->init_mnode_h
#define free_mnode ph->free_mnode_h
#define pt_create ph->pt_create_h
#define pt_insert ph->pt_insert_h
#define pt_search ph->pt_search_h
#define pt_remove ph->pt_remove_h
#define pt_destroy ph->pt_destroy_h
#define init_command_set ph->init_command_set_h
#define add_command ph->add_command_h
#define del_command ph->del_command_h
#define parse_command ph->parse_command_h
#define parse_configuration ph->parse_configuration_h
#define interpret_command ph->interpret_command_h
#define free_cmd_line ph->free_cmd_line_h
#define h_create ph->h_create_h
#define h_insert ph->h_insert_h
#define h_search ph->h_search_h
#define h_remove ph->h_remove_h
#define h_destroy ph->h_destroy_h
#define h_count ph->h_count_h
#define h_iter_begin ph->h_iter_begin_h
#define h_iter_end ph->h_iter_end_h
#define h_iter_next ph->h_iter_next_h
#define h_iter_getkey ph->h_iter_getkey_h
#define h_iter_getvalue ph->h_iter_getvalue_h
#define compare_ipv6 ph->compare_ipv6_h
#define hash_ipv6 ph->hash_ipv6_h
#define set_extent ph->set_extent_h
#define get_extent ph->get_extent_h

/*Extents getters and setters*/
#define set_mnode_extent(m_n, val)										\
	set_extent(true, (void *)m_n, p_id, EXTENT_DEFAULT, (extent_t) val)

#define set_mnode_connector_extent(m_n, val)							\
	set_extent(true, (void *)m_n, p_id, EXTENT_CONNECTOR, (extent_t) val)

#define set_mnode_cipher_extent(m_n, val)								\
	set_extent(true, (void *)m_n, p_id, EXTENT_CIPHER, (extent_t) val)

#define set_mnode_authenticator_extent(m_n, val)						\
	set_extent(true, (void *)m_n, p_id, EXTENT_AUTHENTICATOR, (extent_t) val)

#define set_mserver_extent(m_s, val)									\
	set_extent(false, (void *)m_s, p_id, EXTENT_DEFAULT, (extent_t) val)

#define set_mserver_connector_extent(m_s, val)							\
	set_extent(false, (void *)m_s, p_id, EXTENT_CONNECTOR, (extent_t) val)

#define set_mserver_cipher_extent(m_s, val)								\
	set_extent(false, (void *)m_s, p_id, EXTENT_CIPHER, (extent_t) val)

#define set_mserver_authenticator_extent(m_s, val)						\
	set_extent(false, (void *)m_s, p_id, EXTENT_AUTHENTICATOR, (extent_t) val)

#define get_mnode_extent(m_n)							\
	get_extent(true, (void *)m_n, p_id, EXTENT_DEFAULT)

#define get_mnode_connector_extent(m_n)									\
	get_extent(true, (void *)m_n, ((connector_t *) ph->connectors->ptable[m_n->connector_id])->plugin_id, EXTENT_CONNECTOR)

#define get_mnode_cipher_extent(m_n)									\
	get_extent(true, (void *)m_n, ((cipher_t *) ph->ciphers->ptable[m_n->cipher_id])->plugin_id, EXTENT_CIPHER)

#define get_mnode_authenticator_extent(m_n)								\
	get_extent(true, (void *)m_n, ((authenticator_t *) ph->authenticators->ptable[m_n->authenticator_id])->plugin_id, EXTENT_AUTHENTICATOR)

#define get_mserver_extent(m_s)								\
	get_extent(false, (void *)m_s, p_id, EXTENT_DEFAULT)

#define get_mserver_connector_extent(m_s)								\
	get_extent(false, (void *)m_s, ((connector_t *) ph->connectors->ptable[m_s->connector_id])->plugin_id, EXTENT_CONNECTOR)

#define get_mserver_cipher_extent(m_s)									\
	get_extent(false, (void *)m_s, ((cipher_t *) ph->ciphers->ptable[m_s->cipher_id])->plugin_id, EXTENT_CIPHER)

#define get_mserver_authenticator_extent(m_s)							\
	get_extent(false, (void *)m_s, ((authenticator_t *) ph->authenticators->ptable[m_s->authenticator_id])->plugin_id, EXTENT_AUTHENTICATOR)

/*
 * Logging macros
 */
#ifdef DEBUG

#define ERR(fn)													\
	log_msg(ERROR_LVL,											\
			"%s:%d %s - code %d - %s\n",						\
			__FILE__, __LINE__, fn, errno, strerror(errno));

#define DEBUG_MSG(...) log_msg(DEBUG_LVL, __VA_ARGS__)

#else /* ifndef DEBUG*/

#define ERR(fn)									\
	log_msg(ERROR_LVL,							\
			"%s - code %d - %s\n",				\
			fn, errno, strerror(errno));


#define DEBUG_MSG(...) {}

#endif /*DEBUG*/

#endif
